@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* exec.c: command execution */
-/* (C) 2007-2013 magicant */
+/* (C) 2007-2015 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,9 +38,7 @@
 #include <sys/times.h>
 #include <unistd.h>
 #include <wchar.h>
-#if YASH_ENABLE_ALIAS
-# include "alias.h"
-#endif
+#include "alias.h"
 #include "builtin.h"
 #include "expand.h"
 #if YASH_ENABLE_HISTORY
@@ -145,6 +143,8 @@ static void exec_funcdef(const command_T *c, bool finally_exit)
     __attribute__((nonnull));
 
 static void exec_commands(command_T *c, exec_T type);
+static inline bool should_exit(const command_T *c)
+    __attribute__((nonnull));
 static inline void next_pipe(pipeinfo_T *pi, bool next)
     __attribute__((nonnull));
 static pid_t exec_process(
@@ -197,7 +197,7 @@ pid_t lastasyncpid;
 /* This flag is set to true while the shell is executing the condition of an if-
  * statement, an and-or list, etc. to suppress the effect of the "errexit"
  * option. */
-static bool supresserrexit = false;
+static bool suppresserrexit = false;
 
 /* state of currently executed loop */
 static execstate_T execstate;
@@ -209,6 +209,10 @@ bool special_builtin_executed;
 /* This flag is set while the "exec" built-in is executed. */
 static bool exec_builtin_executed = false;
 
+/* True while executing auxiliary commands such as $PROMPT_COMMAND and
+ * $COMMAND_NOT_FOUND_HANDLER. */
+bool is_executing_auxiliary = false;
+
 /* the last assignment. */
 static const assign_T *last_assign;
 
@@ -292,8 +296,8 @@ void exec_pipelines(const pipeline_T *p, bool finally_exit)
 	if (!first && p->pl_cond == (laststatus != Exit_SUCCESS))
 	    continue;
 
-	bool savesee = supresserrexit;
-	supresserrexit |= p->pl_neg || p->next != NULL;
+	bool savesee = suppresserrexit;
+	suppresserrexit |= p->pl_neg || p->next != NULL;
 
 	bool self = finally_exit && !doing_job_control_now
 	    && !p->next && !p->pl_neg && !any_trap_set;
@@ -305,7 +309,7 @@ void exec_pipelines(const pipeline_T *p, bool finally_exit)
 		laststatus = Exit_SUCCESS;
 	}
 
-	supresserrexit = savesee;
+	suppresserrexit = savesee;
     }
     if (finally_exit)
 	exit_shell();
@@ -381,10 +385,10 @@ bool exec_condition(const and_or_T *c)
     if (c == NULL)
 	return true;
 
-    bool savesee = supresserrexit;
-    supresserrexit = true;
+    bool savesee = suppresserrexit;
+    suppresserrexit = true;
     exec_and_or_lists(c, false);
-    supresserrexit = savesee;
+    suppresserrexit = savesee;
     return laststatus == Exit_SUCCESS;
 }
 
@@ -565,7 +569,6 @@ void exec_commands(command_T *c, exec_T type)
     job_T *job;
     process_T *ps, *pp;
     pipeinfo_T pinfo = PIPEINFO_INIT;
-    commandtype_T lasttype;
 
     /* increment the reference count of `c' to prevent `c' from being freed
      * during execution. */
@@ -585,7 +588,6 @@ void exec_commands(command_T *c, exec_T type)
     do {
 	pid_t pid;
 
-	lasttype = cc->c_type;
 	next_pipe(&pinfo, cc->next != NULL);
 	pid = exec_process(cc,
 		(type == E_SELF && cc->next != NULL) ? E_NORMAL : type,
@@ -641,17 +643,45 @@ void exec_commands(command_T *c, exec_T type)
 	}
     }
 
-    comsfree(c);
-
     handle_signals();
 
-    if (shopt_errexit && !supresserrexit
-	    && laststatus != Exit_SUCCESS && lasttype == CT_SIMPLE
-#if YASH_ENABLE_LINEEDIT
-	    && !(le_state & LE_STATE_COMPLETING)
-#endif
-	    )
+    if (shopt_errexit && should_exit(c))
 	exit_shell_with_status(laststatus);
+
+    comsfree(c);
+}
+
+/* Returns true if the shell should exit because of the `errexit' option. */
+bool should_exit(const command_T *c)
+{
+    if (suppresserrexit)
+	return false;
+    if (laststatus == Exit_SUCCESS)
+	return false;
+
+    /* If this is a multi-command pipeline, the commands are executed in
+     * subshells. Otherwise, we need to check the type of the command. */
+    if (c->next == NULL) {
+	switch (c->c_type) {
+	    case CT_SIMPLE:
+	    case CT_SUBSHELL:
+	    case CT_FUNCDEF:
+		break;
+	    case CT_GROUP:
+	    case CT_IF:
+	    case CT_FOR:
+	    case CT_WHILE:
+	    case CT_CASE:
+		return false;
+	}
+    }
+
+#if YASH_ENABLE_LINEEDIT
+    if (le_state & LE_STATE_COMPLETING)
+	return false;
+#endif
+
+    return true;
 }
 
 /* Updates the contents of the `pipeinfo_T' to proceed to the next process
@@ -953,7 +983,7 @@ pid_t fork_and_reset(pid_t pgid, bool fg, sigtype_T sigtype)
 	if (sigtype & t_leave) {
 	    clear_exit_trap();
 	} else {
-	    clear_traps();
+	    phantomize_traps();
 	    neglect_all_jobs();
 #if YASH_ENABLE_HISTORY
 	    close_history_file();
@@ -1090,8 +1120,7 @@ bool command_not_found_handler(void *const *argv)
     set_positional_parameters(argv);
     set_variable(L VAR_HANDLED, xwcsdup(L""), SCOPE_LOCAL, false);
 
-    result = exec_variable_as_commands(
-	    L VAR_COMMAND_NOT_FOUND_HANDLER, VAR_COMMAND_NOT_FOUND_HANDLER);
+    result = exec_variable_as_auxiliary_(VAR_COMMAND_NOT_FOUND_HANDLER);
     if (result >= 0) {
 	const wchar_t *handledv = getvar(L VAR_HANDLED);
 	handled = (handledv != NULL && handledv[0] != L'\0');
@@ -1234,7 +1263,9 @@ void print_xtrace(void *const *argv)
     bool tracevars = xtrace_buffer.contents != NULL
 		  && xtrace_buffer.length > 0;
 
-    if (shopt_xtrace && (tracevars || argv != NULL)
+    if (shopt_xtrace
+	    && (!is_executing_auxiliary || shopt_traceall)
+	    && (tracevars || argv != NULL)
 #if YASH_ENABLE_LINEEDIT
 	    && !(le_state & LE_STATE_ACTIVE)
 #endif
@@ -1328,8 +1359,7 @@ void exec_function_body(
     (void) complete;
 #endif
     exec_commands(body, finally_exit ? E_SELF : E_NORMAL);
-    if (execstate.exception == E_RETURN)
-	execstate.exception = E_NONE;
+    cancel_return();
     close_current_environment();
 
     execstate.noreturn = save_noreturn;
@@ -1493,6 +1523,17 @@ int exec_variable_as_commands(const wchar_t *varname, const char *codename)
     return result;
 }
 
+/* Calls `exec_variable_as_commands' with `is_executing_auxiliary' set to true.
+ */
+int exec_variable_as_auxiliary(const wchar_t *varname, const char *codename)
+{
+    bool save_executing_auxiliary = is_executing_auxiliary;
+    is_executing_auxiliary = true;
+    int result = exec_variable_as_commands(varname, codename);
+    is_executing_auxiliary = save_executing_auxiliary;
+    return result;
+}
+
 #if YASH_ENABLE_LINEEDIT
 
 /* Autoloads the specified file to load a completion function definition.
@@ -1887,6 +1928,7 @@ int dot_builtin(int argc, void **argv)
     }
 
     exec_input(fd, mbsfilename, enable_alias ? XIO_SUBST_ALIAS : 0);
+    cancel_return();
     remove_shellfd(fd);
     xclose(fd);
     free(mbsfilename);
@@ -2213,7 +2255,6 @@ bool print_command_info(
 	return true;
     }
 
-#if YASH_ENABLE_ALIAS
     if (aliases) {
 	if (print_alias_if_defined(commandname, humanfriendly)) {
 	    return true;
@@ -2222,9 +2263,6 @@ bool print_command_info(
 		return false;
 	}
     }
-#else
-    (void) aliases;
-#endif
 
     name = malloc_wcstombs(commandname);
     if (name == NULL)
