@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* expand.c: word expansion */
-/* (C) 2007-2013 magicant */
+/* (C) 2007-2015 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -107,11 +107,22 @@ static bool try_expand_brace_sequence(
 static bool has_leading_zero(const wchar_t *restrict s, bool *restrict sign)
     __attribute__((nonnull));
 
+static void fieldsplit_all(void **restrict valuelist, void **restrict splitlist,
+	plist_T *restrict dest)
+    __attribute__((nonnull));
 static void fieldsplit(wchar_t *restrict s, char *restrict split,
 	const wchar_t *restrict ifs, plist_T *restrict dest)
     __attribute__((nonnull));
-static void fieldsplit_all(void **restrict valuelist, void **restrict splitlist,
-	plist_T *restrict dest)
+static size_t skip_ifs(const wchar_t *s, const char *split,
+	bool escaped, const wchar_t *ifs)
+    __attribute__((nonnull,pure));
+static size_t skip_ifs_whitespaces(const wchar_t *s, const char *split,
+	bool escaped, const wchar_t *ifs)
+    __attribute__((nonnull,pure));
+static size_t skip_field(const wchar_t *s, const char *split,
+	bool escaped, const wchar_t *ifs)
+    __attribute__((nonnull,pure));
+static void add_empty_field(plist_T *dest, const wchar_t *p)
     __attribute__((nonnull));
 
 static inline void add_sq(
@@ -1386,82 +1397,6 @@ bool has_leading_zero(const wchar_t *restrict s, bool *restrict sign)
 
 /********** Field Splitting **********/
 
-/* Performs field splitting.
- * `s' is the word to split and freed in this function.
- * `split' is the splittability string corresponding to `s' and also freed.
- * The results are added to `dest' as newly-malloced wide strings.
- * `ifs' must not be NULL.
- * The word is split at characters that are contained in `ifs' and whose
- * corresponding character in the splittability string is non-zero. */
-/* Field are split at characters that are contained in `ifs'. If an empty field
- * is split by white spaces, the field is ignored.
- * Split examples (assuming `ifs' = L" -")
- *   " abc 123 "         ->   "abc" "123"
- *   "  abc  123  "      ->   "abc" "123"
- *   "-abc-123-"         ->   "" "abc" "123" ""
- *   " - abc - 123 - "   ->   "" "abc" "123" ""
- *   "abc--123"          ->   "abc" "" "123"
- *   "abc - - 123"       ->   "abc" "" "123"
- */
-void fieldsplit(wchar_t *restrict s, char *restrict split,
-	const wchar_t *restrict ifs, plist_T *restrict dest)
-{
-    size_t index = 0;
-    size_t lwstart = 0, lwend;  /* start/end index of last word */
-
-    for (;;) {
-	/* slip non-IFS characters */
-	lwend = index;
-	if (s[index] == L'\\')
-	    index++;
-	if (s[index] == L'\0')
-	    break;
-	if (!split[index] || wcschr(ifs, s[index]) == NULL) {
-	    index++;
-	    continue;
-	}
-
-	/* the character is in `ifs', so do splitting */
-	bool splitatnonspace = false, nonspace = false;
-	if (lwstart < lwend)
-	    pl_add(dest, xwcsndup(s + lwstart, lwend - lwstart));
-	else
-	    splitatnonspace = true;
-
-	/* skip IFS characters */
-	size_t oldindex;
-	do {
-	    if (!iswspace(s[index])) {
-		if (splitatnonspace)
-		    pl_add(dest, xwcsdup(L""));
-		splitatnonspace = true;
-		nonspace = true;
-	    }
-	    index++;
-	    oldindex = index;
-	    if (s[index] == L'\\')
-		index++;
-	    if (s[index] == L'\0') {
-		if (nonspace)
-		    pl_add(dest, xwcsdup(L""));
-		break;
-	    }
-	} while (split[index] && wcschr(ifs, s[index]) != NULL);
-	lwstart = index = oldindex;
-    }
-    if (lwstart < lwend || lwend == 0) {
-	/* if we have some leftover or the string is empty at all, add it. */
-	if (lwstart > 0) {
-	    xwcsbuf_T buf;
-	    s = wb_towcs(wb_remove(wb_initwith(&buf, s), 0, lwstart));
-	}
-	pl_add(dest, s);
-    } else {
-	free(s);
-    }
-    free(split);
-}
-
 /* Performs field splitting.
  * `valuelist' is a NULL-terminated array of pointers to wide strings to split.
  * `splitlist' is an array of pointers to corresponding splittability strings.
@@ -1484,75 +1419,158 @@ void fieldsplit_all(void **restrict valuelist, void **restrict splitlist,
     free(splitlist);
 }
 
-/* Extracts a field starting at `**sp'.
- * Pointer `*sp' is advanced to the first character of the next field or
- * to the end of string (L'\0') if none. If `**sp' is already L'\0', an empty
- * string is returned.
- * If `ifs' is NULL, the default separator is used.
- * If `noescape' is false, backslashes are treaded as escape characters.
- * The returned string is a newly malloced string. */
-wchar_t *split_next_field(const wchar_t **sp, const wchar_t *ifs, bool noescape)
+/* Performs field splitting.
+ * `s' is the word to split and freed in this function.
+ * `split' is the splittability string corresponding to `s' and also freed.
+ * The results are added to `dest' as newly-malloced wide strings.
+ * `ifs' must not be NULL.
+ * As a special case, if `s' is non-empty but only contains whitespaces from
+ * `ifs', then no field is produced. */
+void fieldsplit(wchar_t *restrict s, char *restrict split,
+	const wchar_t *restrict ifs, plist_T *restrict dest)
 {
-    size_t i = 0, length = 0;
-    const wchar_t *s = *sp;
-    wchar_t *result;
+    plist_T fields;
 
-    if (ifs == NULL)
-	ifs = DEFAULT_IFS;
-    while (*s != L'\0' && wcschr(ifs, *s) != NULL && iswspace(*s))
-	s++;
-    if (*s != L'\0' && wcschr(ifs, *s) != NULL && !iswspace(*s)) {
-	assert(length == 0);
-	i = 1;
-	goto end;
-    }
-    while (s[i] != L'\0') {
-	if (!noescape && s[i] == L'\\') {
-	    i++;
-	    if (s[i] == L'\0') {
-		length = i;
-		break;
-	    }
-	    i++;
-	} else if (wcschr(ifs, s[i]) != NULL) {
-	    length = i;
-	    do {
-		if (!iswspace(s[i])) {
-		    i++;
-		    while (wcschr(ifs, s[i]) != NULL && iswspace(s[i]))
-			i++;
-		    break;
-		}
-	    } while (wcschr(ifs, s[++i]) != NULL);
-	    break;
-	} else {
-	    i++;
-	    length = i;
-	}
+    pl_init(&fields);
+    extract_fields(s, split, true, ifs, &fields);
+    assert(fields.length % 2 == 0);
+
+    if (s[0] != L'\0' && fields.length == 2 &&
+	    fields.contents[0] == fields.contents[1])
+	pl_clear(&fields, 0);
+
+    for (size_t i = 0; i < fields.length; i += 2) {
+	const wchar_t *start = fields.contents[i], *end = fields.contents[i+1];
+	pl_add(dest, xwcsndup(start, end - start));
     }
 
-end:
-    result = xwcsndup(s, length);
-    if (!noescape)
-	result = unescapefree(result);
-    *sp = s + i;
-    return result;
+    pl_destroy(&fields);
+    free(s);
+    free(split);
 }
 
-/* Removes IFS white spaces at the end of the string `s' by replacing them with
- * null characters. The default IFS is used if `ifs' is NULL. */
-void trim_trailing_spaces(wchar_t *restrict s, const wchar_t *restrict ifs)
+/* Extracts fields from a string.
+ * `s' is the word to split.
+ * `split' is the splittability string corresponding to `s'. It must be at least
+ * as long as `wcslen(s)'.
+ * If `escaped' is true, backslashes in `s' are treated as escapes. But
+ * backslashes do not prevent splitting.
+ * `ifs' must not be NULL.
+ *
+ * The results are appended to `dest'. If n fields are found, 2n pointers are
+ * appended to `dest'. The first pointer points to the first character of the
+ * first field in `s'. The second to the character past the last character of
+ * the first field. The third to the first character of the second field. And so
+ * on.
+ *
+ * The word is split at characters that are contained in `ifs' and whose
+ * corresponding character in the splittability string is non-zero. Refer to
+ * POSIX for how whitespaces are treated in field splitting.
+ *
+ * If the last field is empty, the index is taken just after the last character
+ * that is not an IFS-whitespace. */
+/* Split examples (assuming `ifs' = L" -")
+ *   ""                  ->   ""
+ *   "  "                ->   ""
+ *   " abc 123 "         ->   "abc" "123"
+ *   "  abc  123  "      ->   "abc" "123"
+ *   "-abc-123-"         ->   "" "abc" "123" ""
+ *   " - abc - 123 - "   ->   "" "abc" "123" ""
+ *   "abc--123"          ->   "abc" "" "123"
+ *   "abc - - 123"       ->   "abc" "" "123"
+ */
+void extract_fields(const wchar_t *restrict s, const char *restrict split,
+	bool escaped, const wchar_t *restrict ifs, plist_T *restrict dest)
 {
-    if (ifs == NULL)
-	ifs = DEFAULT_IFS;
-    for (size_t i = wcslen(s); i-- > 0; ) {
-	if (wcschr(ifs, s[i]) != NULL && iswspace(s[i]))
-	    s[i] = L'\0';
-	else
-	    break;
+    size_t index = 0;
+
+    /* true when the currently skipping IFS whitespaces immediately follow a
+     * previously split field. */
+    bool afterfield = false;
+
+    for (;;) {
+	size_t ifswhitestartindex = index;
+	index += skip_ifs_whitespaces(&s[index], &split[index], escaped, ifs);
+
+	/* extract next field, if any */
+	size_t fieldstartindex = index;
+	index += skip_field(&s[index], &split[index], escaped, ifs);
+	if (index != fieldstartindex) {
+	    pl_add(pl_add(dest, &s[fieldstartindex]), &s[index]);
+	    afterfield = true;
+	    continue;
+	}
+
+	/* Now the current char is either null or a IFS non-whitespace. */
+
+	if (!afterfield)
+	    add_empty_field(dest, &s[ifswhitestartindex]);
+
+	/* skip (only one) IFS non-whitespace */
+	size_t ifsstartindex = index;
+	index += skip_ifs(&s[index], &split[index], escaped, ifs);
+	if (index != ifsstartindex) {
+	    afterfield = false;
+	    continue;
+	}
+
+	/* Now the current char is null. We're done. */
+	return;
     }
 }
 
+/* If `*s' is a (possibly escaped if `escaped') IFS character, returns the
+ * number of characters to skip it. Otherwise returns zero. */
+size_t skip_ifs(const wchar_t *s, const char *split,
+	bool escaped, const wchar_t *ifs)
+{
+    size_t i = 0;
+    if (escaped && s[i] == L'\\')
+	i++;
+    if (s[i] == L'\0')
+	return 0;
+    if (split[i] && wcschr(ifs, s[i]) != NULL)
+	return i + 1;
+    else
+	return 0;
+}
+
+/* Returns the length of IFS whitespace sequence starting at `*s'. */
+size_t skip_ifs_whitespaces(const wchar_t *s, const char *split,
+	bool escaped, const wchar_t *ifs)
+{
+    size_t total = 0;
+    for (;;) {
+	size_t current = skip_ifs(&s[total], &split[total], escaped, ifs);
+	if (current == 0 || !iswspace(s[total + current - 1]))
+	    return total;
+	total += current;
+    }
+}
+
+/* Returns the length of a field starting at `*s'. */
+size_t skip_field(const wchar_t *s, const char *split,
+	bool escaped, const wchar_t *ifs)
+{
+    size_t index = 0;
+    for (;;) {
+	size_t saveindex = index;
+	if (escaped && s[index] == L'\\')
+	    index++;
+	if (s[index] == L'\0')
+	    return saveindex;
+	if (split[index] && wcschr(ifs, s[index]) != NULL)
+	    return saveindex;
+	index++;
+    }
+}
+
+void add_empty_field(plist_T *dest, const wchar_t *p)
+{
+    pl_add(dest, p);
+    pl_add(dest, p);
+}
+
 
 /********** Escaping **********/
 
@@ -1772,9 +1790,7 @@ wchar_t *parse_and_expand_string(const wchar_t *s, const char *name, bool esc)
     parseparam_T info = {
 	.print_errmsg = true,
 	.enable_verbose = false,
-#if YASH_ENABLE_ALIAS
 	.enable_alias = true,
-#endif
 	.filename = name,
 	.lineno = 1,
 	.input = input_wcs,
