@@ -27,6 +27,7 @@
 #if HAVE_GETTEXT
 # include <libintl.h>
 #endif
+#include <limits.h>
 #if HAVE_PATHS_H
 # include <paths.h>
 #endif
@@ -388,13 +389,16 @@ char *which(
 
 /* Creates a new temporary file under "/tmp".
  * `mode' is the access permission bits of the file.
+ * The name of the temporary file ends with `suffix`, which should be as short
+ * as possible and must not exceed 8 bytes.
  * On successful completion, a file descriptor is returned, which is both
  * readable and writeable regardless of `mode', and a pointer to a string
  * containing the filename is assigned to `*filename', which should be freed by
  * the caller. The filename consists only of portable filename characters.
  * On failure, -1 is returned with `**filename' left unchanged and `errno' is
  * set to the error value. */
-int create_temporary_file(char **filename, mode_t mode)
+int create_temporary_file(
+	char **restrict filename, const char *restrict suffix, mode_t mode)
 {
     static uintmax_t num = 0;
     uintmax_t n;
@@ -407,8 +411,13 @@ int create_temporary_file(char **filename, mode_t mode)
     sb_init(&buf);
     for (int i = 0; i < 100; i++) {
 	num = (num ^ n) * 16777619;
-	sb_printf(&buf, "/tmp/yash-%X", (unsigned) (num >> 30));
-	/* The filename must be 14 bytes long at most. */
+	sb_printf(&buf, "/tmp/yash-%" PRIXMAX, num);
+
+	size_t maxlen = _POSIX_NAME_MAX + 5 - strlen(suffix);
+	if (buf.length > maxlen)
+	    sb_truncate(&buf, maxlen);
+	sb_cat(&buf, suffix);
+
 	fd = open(buf.contents, O_RDWR | O_CREAT | O_EXCL, mode);
 	if (fd >= 0) {
 	    *filename = sb_tostr(&buf);
@@ -1286,7 +1295,7 @@ step10:  /* do chdir */
 	}
     }
     if (!posixly_correct)
-	exec_variable_as_commands(L VAR_YASH_AFTER_CD, VAR_YASH_AFTER_CD);
+	exec_variable_as_auxiliary_(VAR_YASH_AFTER_CD);
 
     return Exit_SUCCESS;
 }
